#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>


/*
 ***** DICHIARAZIONE FUNZIONI *****
 */
void gpk_game_reset(GtkButton *button);
void gpk_dump_chessboard(GtkButton *button);
void gpk_dump_valid_moves(void);
void gpk_update_chessboard(GtkButton *button);
void gpk_update_stat_num(GtkButton *button, int num, int val);
void gpk_update_stat_str(GtkButton *button, int num, char *str);
void gpk_update_stats(GtkButton *button);
void gpk_actor_game(int x, int y, GtkWidget *button);
void gpk_set_checkboard(GtkButton *button, int x, int y, gchar val, int colore);
void gpk_update_num_chips(int who, gchar type);
void gpk_check_gameover(GtkButton *button);
void gpk_backup_colors(void);
void gpk_alternate_colors(void);
void gpk_recovery_colors(void);
void gpk_automata(void);
void gpk_do_arrocco(int who);
int gpk_find_chips(int actual_player);

void gpk_insert_best_move(int x, int y);
int gpk_compute_best_moves(int x, int y);
void gpk_insert_valid_move(int x, int y);
int gpk_compute_valid_moves(int x, int y);
int gpk_check_this_move(int x, int y);

void gpk_thread(void *arg);


/*
 ***** VARIABILI DI GIOCO *****
 */
gchar gt[8][12]; // Matrice pedine sulla scacchiera (8x8 + puntatori esterni x arrocco)
int gc[8][12]; // Matrice colore pedine sulla scacchiera (8x8 + puntatori esterni x arrocco)
int gc2[8][12]; // Matrice colore pedine sulla scacchiera (temp x suggerimenti mosse)
int gn1[8]; // Contatore pedine PLAYER 1
int gn2[8]; // Contatore pedine PLAYER 2
int vittorie[2]; // Contatore vittorie

int stato=0; // Stato del gioco
int mossa[2]; // Memoria mossa
int num_mosse=0; // Numero di mosse nella partita attuale
int actual_player=1; // Giocatore di mano
int arrocco_flag[2]; //E' possibile l'arrocco (k e t non mossi) ?
int arrocco_started; //E' da avviare l'arrocco (attendere richiesta) ?

int best_move[28][2];  //Mosse buone per la posizione attuale
int valid_move[28][2]; //Mosse valide per la posizione attuale
int valid_chip[28][2]; //Mosse valide per la posizione attuale
int best_move_cnt=0;  //Numero di mosse buone nella mano attuale
int valid_move_cnt=0; //Numero di mosse valide nella mano attuale
int valid_chip_cnt=0; //Numero di pedine sulla scacchiera

int thread_started=0; //Numero di thread supplementari avviati
pthread_t thd; //PTread Structures


/*
 ***** DEFINIZIONI FUNZIONI *****
 */

/* ***** AGGIORNA 1 STAT NUMERICA ***** */
void gpk_update_stat_num(GtkButton *button, int num, int val)
{
	gchar temp[2];
	gchar target[10];
	
	sprintf(temp, "%d", val);
	sprintf(target, "entry%d", num);
	GtkWidget* gwidget= lookup_widget(GTK_WIDGET(button), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
}


/* ***** AGGIORNA 1 STAT TESTUALE ***** */
void gpk_update_stat_str(GtkButton *button, int num, char *str)
{
	gchar target[10];
	
	sprintf(target, "entry%d", num);
	GtkWidget* gwidget= lookup_widget(GTK_WIDGET(button), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), str);
}


/* ***** AGGIORNA STATISTICHE ***** */
void gpk_update_stats(GtkButton *button)
{
	gpk_update_stat_num(button, 13, gn1[1]);
	gpk_update_stat_num(button, 14, gn1[2]);
	gpk_update_stat_num(button, 15, gn1[3]);
	gpk_update_stat_num(button, 16, gn1[4]);
	gpk_update_stat_num(button, 17, gn1[5]);

	gpk_update_stat_num(button, 18, gn2[1]);
	gpk_update_stat_num(button, 19, gn2[2]);
	gpk_update_stat_num(button, 20, gn2[3]);
	gpk_update_stat_num(button, 21, gn2[4]);
	gpk_update_stat_num(button, 22, gn2[5]);

	gpk_update_stat_num(button, 1, vittorie[0]);
	gpk_update_stat_num(button, 3, vittorie[1]);
	gpk_update_stat_num(button, 11, stato);
	gpk_update_stat_num(button, 7, actual_player);
	gpk_update_stat_num(button, 10, num_mosse);
}


/* ***** CONTA LE PEDINE ***** */
void gpk_update_num_chips(int who, gchar type) {
	if (who==2) {
		if (type=='K') gn2[0]--;
		if (type=='Q') gn2[1]--;
		if (type=='B') gn2[2]--;
		if (type=='H') gn2[3]--;
		if (type=='T') gn2[4]--;
		if (type=='p') gn2[5]--;
	} else {
		if (type=='K') gn1[0]--;
		if (type=='Q') gn1[1]--;
		if (type=='B') gn1[2]--;
		if (type=='H') gn1[3]--;
		if (type=='T') gn1[4]--;
		if (type=='p') gn1[5]--;
	}
}


/* ***** CONTA LE PEDINE ***** */
void gpk_check_gameover(GtkButton *button) {
	if (gn1[0]==0) {
		vittorie[0]++;
		stato=0;
		gpk_game_reset(button);
	}
	if (gn2[0]==0) {
		vittorie[1]++;
		stato=0;
		gpk_game_reset(button);
	}
}


/* ***** AUTOMATISMI DI GIOCO (promozione pedone) ***** */
void gpk_automata(void) {

	int i;

	for (i=0;i<8;i++) { // ----- PROMOZIONE PEDONE ROSSO -----
		if ((gt[i][0]=='p')&&(gc[i][0]==2)) {
			gt[i][0]='Q';
		}
	}
	for (i=0;i<8;i++) { // ----- PROMOZIONE PEDONE NERO -----
		if ((gt[i][7]=='p')&&(gc[i][7]==1)) {
			gt[i][7]='Q';
		}
	}
}


/* ***** ESEGUE L'ARROCCO ***** */
void gpk_do_arrocco(int who) {

	if (who==2) { // ----- ARROCCO ROSSO -----
		//gt[6][7]=gt[4][7];
		//gt[5][7]=gt[7][7];
		gt[4][7]=' ';
		gt[5][7]='T';
		gt[6][7]='K';
		gt[7][7]=' ';
		gc[4][7]=0;
		gc[5][7]=2;
		gc[6][7]=2;
		gc[7][7]=0;
	}
	if (who==1) { // ----- ARROCCO NERO -----
		//gt[6][0]=gt[4][0];
		//gt[5][0]=gt[7][0];
		gt[4][0]=' ';
		gt[5][0]='T';
		gt[6][0]='K';
		gt[7][0]=' ';
		gc[4][0]=0;
		gc[5][0]=1;
		gc[6][0]=1;
		gc[7][0]=0;
	}
}


/* ***** INSERISCE UNA NUOVA MOSSA POSSIBILE NELLA LISTA ***** */
void gpk_insert_valid_move(int x, int y) {
	valid_move[valid_move_cnt][0]=x;
	valid_move[valid_move_cnt][1]=y;
	valid_move_cnt++;
}


/* ***** INSERISCE UNA NUOVA BUONA MOSSA POSSIBILE NELLA LISTA ***** */
void gpk_insert_best_move(int x, int y) {
	best_move[valid_move_cnt][0]=x;
	best_move[valid_move_cnt][1]=y;
	best_move_cnt++;
}


/* ***** INSERISCE UNA NUOVA MOSSA POSSIBILE NELLA LISTA ***** */
int gpk_find_chips(int actual_player) {

	int i,j,cnt;

	cnt=0;
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if ((gt[i][j]!=' ')&&(gc[i][j]==actual_player)) {
				valid_chip[cnt][0]=i;
				valid_chip[cnt][1]=j;
				cnt++;
			}
		}
	}
	return cnt;
}


/* ***** DUMP MATRICE MOSSE VALIDE ***** */
void gpk_dump_valid_moves(void) {

	int i;
	gchar temp[30];

	sprintf(temp, "Dump mosse valide: %d\n", valid_move_cnt);
	g_print (temp);
	for (i=0;i<valid_move_cnt;i++) {
		sprintf(temp, "[%d]-[%d]\n", valid_move[i][0], valid_move[i][1]);
		g_print (temp);
	}
	g_print ("\n");
}


/* ***** SALVA COLORI NELLA MATRICE TEMPORANEA ***** */
void gpk_backup_colors(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc2[i][j]=gc[i][j];
		}
	}
}


/* ***** REIPRISTINA COLORI DALLA MATRICE TEMPORANEA ***** */
void gpk_recovery_colors(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc[i][j]=gc2[i][j];
		}
	}
}


/* ***** REIPRISTINA COLORI DALLA MATRICE TEMPORANEA ***** */
void gpk_alternate_colors(void) {

	int i;

	for (i=0;i<valid_move_cnt;i++) {
		if (valid_move[i][1]==10) {        // ----- ARROCCO NERO -----
			gc[4][0]=4;
			gc[5][0]=4;
			gc[6][0]=4;
			gc[7][0]=4;
			arrocco_started=1;
		} else if (valid_move[i][1]==11) { // ----- ARROCCO ROSSO -----
			gc[4][7]=4;
			gc[5][7]=4;
			gc[6][7]=4;
			gc[7][7]=4;
			arrocco_started=1;
		} else {                           // ----- MOSSE NORMALI -----
			if (gc[valid_move[i][0]][valid_move[i][1]]==0) {
				gc[valid_move[i][0]][valid_move[i][1]]=3; // colora di giallo le mosse possibili
			} else {
				gc[valid_move[i][0]][valid_move[i][1]]=5; // colora di verde quando si mangia
			}
		}
	}
}


/* ***** CONTROLLA SE UNA MOSSA E' VALIDA ***** */
int gpk_check_this_move(int x, int y) {

	int i;
	
	for (i=0;i<28;i++) {
		if ((valid_move[i][0]==x)&&(valid_move[i][1]==y)) {
			return 1;
		}
	}
	return 0;
}


/* ***** DUMP MATRICE SCACCHIERA ***** */
void gpk_dump_chessboard(GtkButton *button) {
	
	int i,j;
	gchar temp[30];
	
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			sprintf(temp, "%d-", gt[i][j]);
			g_print (temp);
		}
		g_print ("\n");
	}
	g_print ("\n");
	sprintf(temp, "Stato: %d\n", stato);
	g_print (temp);
}


/* ***** AGGIORNA SCACCHIERA ***** */
void gpk_update_chessboard(GtkButton *button) {
	
	int i,j;
	gchar temp[30];
	
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gpk_set_checkboard(button, i, j, gt[i][j], gc[i][j]);
		}
	}
}


/* ***** RESET GIOCO ***** */
void gpk_game_reset(GtkButton *button) {

	// INIT
	int i,j;
	gtk_rc_parse("/home/gipoco/Projects/project1/src/joshua.rc");
	stato=0;
	arrocco_flag[0]=1;
	arrocco_flag[1]=1;
	arrocco_started=0;
	actual_player=1;
	num_mosse=0;
	
	// RESET NUMERO PEDINE
	gn1[0]=1;
	gn1[1]=1;
	gn1[2]=2;
	gn1[3]=2;
	gn1[4]=2;
	gn1[5]=8;
	gn2[0]=1;
	gn2[1]=1;
	gn2[2]=2;
	gn2[3]=2;
	gn2[4]=2;
	gn2[5]=8;

	// RESET POSIZIONI E COLORI
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gt[i][j]=' ';
			gc[i][j]=0;
		}
	}		
	
	// METTI PEDINE IN MEMORIA
	gt[0][0]='T';
	gt[1][0]='H';
	gt[2][0]='B';
	gt[3][0]='Q';
 	gt[4][0]='K';
	gt[5][0]='B';
	gt[6][0]='H';
	gt[7][0]='T';
	for (i=0; i<8; i++) {
		gt[i][1]='p';
		gt[i][6]='p';
	}
	gt[0][7]='T';
	gt[1][7]='H';
	gt[2][7]='B';
	gt[3][7]='Q';
	gt[4][7]='K';
	gt[5][7]='B';
	gt[6][7]='H';
	gt[7][7]='T';

	// METTI COLORI IN MEMORIA
	for (i=0; i<8; i++) {
		gc[i][0]=1;
		gc[i][1]=1;
		gc[i][6]=2;
		gc[i][7]=2;
	}

	// STAMPA MESSAGGIO
	gpk_update_stat_str(button, 24, "Choose num players");
}


/* ***** CALCOLA E COLORA MOSSE VALIDE DA UNA POSIZIONE DATA ***** */
int gpk_compute_valid_moves(int x, int y) {

	int i,j,flag;
	int my_color=((gc[x][y]==1)?1:2);
	int his_color=((gc[x][y]==1)?2:1);

	valid_move_cnt=0;

	for (i=0;i<28;i++) {
		valid_move[i][0]=0;
		valid_move[i][1]=0;
	}

	if (gt[x][y]=='K') { // ------------------- R E -----------------------
		
		// --- normali ---
		if ((x<7)&&((gc[x+1][y]==his_color)||(gt[x+1][y]==' '))) gpk_insert_valid_move(x+1, y);
		if ((x>0)&&((gc[x-1][y]==his_color)||(gt[x-1][y]==' '))) gpk_insert_valid_move(x-1, y);
		if ((y>0)&&((gc[x][y-1]==his_color)||(gt[x][y-1]==' '))) gpk_insert_valid_move(x, y-1);
		if ((y<7)&&((gc[x][y+1]==his_color)||(gt[x][y+1]==' '))) gpk_insert_valid_move(x, y+1);

		// --- diagonali ---
		if ((x<7)&&(y<7)&&((gc[x+1][y+1]==his_color)||(gt[x+1][y+1]==' '))) gpk_insert_valid_move(x+1, y+1);
		if ((x>0)&&(y>0)&&((gc[x-1][y-1]==his_color)||(gt[x-1][y-1]==' '))) gpk_insert_valid_move(x-1, y-1);
		if ((x<7)&&(y>0)&&((gc[x+1][y-1]==his_color)||(gt[x+1][y-1]==' '))) gpk_insert_valid_move(x+1, y-1);
		if ((x>0)&&(y<7)&&((gc[x-1][y+1]==his_color)||(gt[x-1][y+1]==' '))) gpk_insert_valid_move(x-1, y+1);

		// --- arrocco NERO ---
		if ((arrocco_flag[0])&&(gt[5][0]==' ')&&(gt[6][0]==' ')) gpk_insert_valid_move(6, 10);

		// --- arrocco ROSSO ---
		if ((arrocco_flag[1])&&(gt[5][7]==' ')&&(gt[6][7]==' ')) gpk_insert_valid_move(6, 11);
	}

	if (gt[x][y]=='H') { // ------------------- C A V A L L O -----------------------
		
		if ((x<7)&&(y<6)&&((gc[x+1][y+2]==his_color)||(gt[x+1][y+2]==' '))) gpk_insert_valid_move(x+1, y+2);
		if ((x<6)&&(y<7)&&((gc[x+2][y+1]==his_color)||(gt[x+2][y+1]==' '))) gpk_insert_valid_move(x+2, y+1);
		if ((x>0)&&(y>1)&&((gc[x-1][y-2]==his_color)||(gt[x-1][y-2]==' '))) gpk_insert_valid_move(x-1, y-2);
		if ((x>1)&&(y>0)&&((gc[x-2][y-1]==his_color)||(gt[x-2][y-1]==' '))) gpk_insert_valid_move(x-2, y-1);
		if ((x>0)&&(y<6)&&((gc[x-1][y+2]==his_color)||(gt[x-1][y+2]==' '))) gpk_insert_valid_move(x-1, y+2);
		if ((x<6)&&(y>0)&&((gc[x+2][y-1]==his_color)||(gt[x+2][y-1]==' '))) gpk_insert_valid_move(x+2, y-1);
		if ((x<7)&&(y>1)&&((gc[x+1][y-2]==his_color)||(gt[x+1][y-2]==' '))) gpk_insert_valid_move(x+1, y-2);
		if ((x>1)&&(y<7)&&((gc[x-2][y+1]==his_color)||(gt[x-2][y+1]==' '))) gpk_insert_valid_move(x-2, y+1);
	}

	if (gt[x][y]=='p') { // ------------------- P E D O N E -----------------------
		
		if (gc[x][y]==1) { // ----- NERO -----
			if ((y<7)&&(gt[x][y+1]==' '))        gpk_insert_valid_move(x, y+1);   // scende
			if ((y==1)&&(gt[x][3]==' '))         gpk_insert_valid_move(x, 3);     // subito salta 2
			if ((x<7)&&(y<7)&&(gc[x+1][y+1]==2)) gpk_insert_valid_move(x+1, y+1); // mangia a destra
			if ((x>0)&&(y<7)&&(gc[x-1][y+1]==2)) gpk_insert_valid_move(x-1, y+1); // mangia a sinistra
		}
		if (gc[x][y]==2) { // ----- ROSSO -----
			if ((y>0)&&(gt[x][y-1]==' '))        gpk_insert_valid_move(x, y-1);   // sale
			if ((y==6)&&(gt[x][4]==' '))         gpk_insert_valid_move(x, 4);     // subito salta 2
			if ((x<7)&&(y>0)&&(gc[x+1][y-1]==1)) gpk_insert_valid_move(x+1, y-1); // mangia a destra
			if ((x>0)&&(y>0)&&(gc[x-1][y-1]==1)) gpk_insert_valid_move(x-1, y-1); // mangia a sinistra
		}
	}

	if ((gt[x][y]=='T')||(gt[x][y]=='Q')) { // ------------------- T O R R E  /  R E G I N A -----------------------

		// -- scende --
		flag=0; i=y+1;
		while ((flag==0)&&(i<8)) {
			if (gt[x][i]==' ') gpk_insert_valid_move(x, i);
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_valid_move(x, i);
				flag++;
			}
			i++;
		}

		// -- sale --
		flag=0; i=y-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[x][i]==' ') gpk_insert_valid_move(x, i);
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_valid_move(x, i);
				flag++;
			}
			i--;
		}

		// -- slide DX --
		flag=0; i=x+1;
		while ((flag==0)&&(i<8)) {
			if (gt[i][y]==' ') gpk_insert_valid_move(i, y);
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_valid_move(i, y);
				flag++;
			}
			i++;
		}

		// -- slide SX --
		flag=0; i=x-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[i][y]==' ') gpk_insert_valid_move(i, y);
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_valid_move(i, y);
				flag++;
			}
			i--;
		}
	}

	if ((gt[x][y]=='B')||(gt[x][y]=='Q')) { // ------------------- A L F I E R E  /  R E G I N A -----------------------

		// -- DOWN DX --
		flag=0; i=x+1; j=y+1;
		while ((flag==0)&&(i<8)&&(j<8)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i++; j++;
		}

		// -- DOWN SX --
		flag=0; i=x-1; j=y+1;
		while ((flag==0)&&(i>=0)&&(j<8)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i--; j++;
		}

		// -- UP SX --
		flag=0; i=x-1; j=y-1;
		while ((flag==0)&&(i>=0)&&(j>=0)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i--; j--;
		}

		// -- UP DX --
		flag=0; i=x+1; j=y-1;
		while ((flag==0)&&(i<8)&&(j>=0)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i++; j--;
		}
	}

	return valid_move_cnt;
}


/* ***** CERCA MOSSE MIGLIORI DA UNA POSIZIONE DATA ***** */
int gpk_compute_best_moves(int x, int y) {

	int i,j,flag;
	int my_color=((gc[x][y]==1)?1:2);
	int his_color=((gc[x][y]==1)?2:1);

	best_move_cnt=0;

	for (i=0;i<28;i++) {
		best_move[i][0]=0;
		best_move[i][1]=0;
	}

	if (gt[x][y]=='K') { // ------------------- R E -----------------------
		
		// --- normali ---
		if ((x<7)&&(gc[x+1][y]==his_color)) gpk_insert_best_move(x+1, y);
		if ((x>0)&&(gc[x-1][y]==his_color)) gpk_insert_best_move(x-1, y);
		if ((y>0)&&(gc[x][y-1]==his_color)) gpk_insert_best_move(x, y-1);
		if ((y<7)&&(gc[x][y+1]==his_color)) gpk_insert_best_move(x, y+1);

		// --- diagonali ---
		if ((x<7)&&(y<7)&&(gc[x+1][y+1]==his_color)) gpk_insert_best_move(x+1, y+1);
		if ((x>0)&&(y>0)&&(gc[x-1][y-1]==his_color)) gpk_insert_best_move(x-1, y-1);
		if ((x<7)&&(y>0)&&(gc[x+1][y-1]==his_color)) gpk_insert_best_move(x+1, y-1);
		if ((x>0)&&(y<7)&&(gc[x-1][y+1]==his_color)) gpk_insert_best_move(x-1, y+1);
	}

	if (gt[x][y]=='H') { // ------------------- C A V A L L O -----------------------
		
		if ((x<7)&&(y<6)&&(gc[x+1][y+2]==his_color)) gpk_insert_best_move(x+1, y+2);
		if ((x<6)&&(y<7)&&(gc[x+2][y+1]==his_color)) gpk_insert_best_move(x+2, y+1);
		if ((x>0)&&(y>1)&&(gc[x-1][y-2]==his_color)) gpk_insert_best_move(x-1, y-2);
		if ((x>1)&&(y>0)&&(gc[x-2][y-1]==his_color)) gpk_insert_best_move(x-2, y-1);
		if ((x>0)&&(y<6)&&(gc[x-1][y+2]==his_color)) gpk_insert_best_move(x-1, y+2);
		if ((x<6)&&(y>0)&&(gc[x+2][y-1]==his_color)) gpk_insert_best_move(x+2, y-1);
		if ((x<7)&&(y>1)&&(gc[x+1][y-2]==his_color)) gpk_insert_best_move(x+1, y-2);
		if ((x>1)&&(y<7)&&(gc[x-2][y+1]==his_color)) gpk_insert_best_move(x-2, y+1);
	}

	if (gt[x][y]=='p') { // ------------------- P E D O N E -----------------------
		
		if (gc[x][y]==1) { // ----- NERO -----
			if ((x<7)&&(y<7)&&(gc[x+1][y+1]==2)) gpk_insert_best_move(x+1, y+1); // mangia a destra
			if ((x>0)&&(y<7)&&(gc[x-1][y+1]==2)) gpk_insert_best_move(x-1, y+1); // mangia a sinistra
		}
		if (gc[x][y]==2) { // ----- ROSSO -----
			if ((x<7)&&(y>0)&&(gc[x+1][y-1]==1)) gpk_insert_best_move(x+1, y-1); // mangia a destra
			if ((x>0)&&(y>0)&&(gc[x-1][y-1]==1)) gpk_insert_best_move(x-1, y-1); // mangia a sinistra
		}
	}

	if ((gt[x][y]=='T')||(gt[x][y]=='Q')) { // ------------------- T O R R E  /  R E G I N A -----------------------

		// -- scende --
		flag=0; i=y+1;
		while ((flag==0)&&(i<8)) {
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_best_move(x, i);
				flag++;
			}
			i++;
		}

		// -- sale --
		flag=0; i=y-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_best_move(x, i);
				flag++;
			}
			i--;
		}

		// -- slide DX --
		flag=0; i=x+1;
		while ((flag==0)&&(i<8)) {
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_best_move(i, y);
				flag++;
			}
			i++;
		}

		// -- slide SX --
		flag=0; i=x-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_best_move(i, y);
				flag++;
			}
			i--;
		}
	}

	if ((gt[x][y]=='B')||(gt[x][y]=='Q')) { // ------------------- A L F I E R E  /  R E G I N A -----------------------

		// -- DOWN DX --
		flag=0; i=x+1; j=y+1;
		while ((flag==0)&&(i<8)&&(j<8)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i++; j++;
		}

		// -- DOWN SX --
		flag=0; i=x-1; j=y+1;
		while ((flag==0)&&(i>=0)&&(j<8)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i--; j++;
		}

		// -- UP SX --
		flag=0; i=x-1; j=y-1;
		while ((flag==0)&&(i>=0)&&(j>=0)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i--; j--;
		}

		// -- UP DX --
		flag=0; i=x+1; j=y-1;
		while ((flag==0)&&(i<8)&&(j>=0)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i++; j--;
		}
	}

	return valid_move_cnt;
}


/* ***** CONTROLLA ED EFFETTUA MOSSA ***** */
void gpk_actor_game(int x, int y, GtkWidget *button) {
	
	int changed=0;
	
	if ((stato==0)||(stato==10)||(stato==20)||(stato==22)) {                      // PRIMA GUANTO LA PEDINA.....
		if (stato==10) gpk_compute_best_moves(x,y); // Se gioca da solo cerca mosse furbe, e cque tutte le valide
		if (gpk_compute_valid_moves(x,y)>0) {
			//gpk_dump_valid_moves();
			mossa[0]=x;
			mossa[1]=y;
			gpk_backup_colors();
			gpk_alternate_colors();
			stato++;
			changed++;
		}
	}
	if (((stato==1)||(stato==11)||(stato==21)||(stato==23))&&(changed==0)) {      // POI LA RIMETTO SULLA SCACCHIERA
		if (stato==23) usleep(500000);
		gpk_recovery_colors();
		if (arrocco_started) {
			if ((x==6)&&(y==0)) y=10;
			if ((x==6)&&(y==7)) y=11;
			arrocco_started=0;
		}
		if (gpk_check_this_move(x,y)) {
			if (y>9) {                          // se ha richiesto l-arrocco fa la procedura speciale
				gpk_do_arrocco(gc[mossa[0]][mossa[1]]);
			} else {                            // altrimenti esegue una mossa normale
				if (gt[mossa[0]][mossa[1]]=='K') arrocco_flag[(gc[mossa[0]][mossa[1]]==2)?1:0]=0;
				if (gc[x][y]>0) gpk_update_num_chips((gc[mossa[0]][mossa[1]]), gt[x][y]);
				gt[x][y]=gt[mossa[0]][mossa[1]];
				gc[x][y]=gc[mossa[0]][mossa[1]];
				gt[mossa[0]][mossa[1]]=' ';
				gc[mossa[0]][mossa[1]]=0;
			}
		} else {
			if (stato==21) stato--;
			actual_player++;
		}
		if (stato==20) {
			//stato++;
		} else if (stato==21) {
			stato++;
		} else if (stato==23) {
			stato=20;
		} else {
			stato--;
		}
		gpk_automata();
		num_mosse++;
		actual_player++;
		if (actual_player>2) actual_player=1;
	}
	gpk_update_stats(button);
	gpk_check_gameover(button);
	gpk_update_chessboard(((GtkButton *)(button)));
	//gtk_widget_focus_out(button); // ----------------- Toglie la selezione BLU dalla scacchiera ------------------
}


/* ***** Wrapper scrittura carattere sulla scacchiera ***** */
void gpk_set_checkboard(GtkButton *button, int x, int y, gchar val, int colore) {
	
	gchar temp[2];
	gchar target[10];
	
	sprintf(temp, "%c", val);
	sprintf(target, "entry%d", (short)(26+x+(y*8)));
	GtkWidget* gwidget= lookup_widget(GTK_WIDGET(button), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
	if (colore==0) { gtk_widget_set_name (gwidget, "grigio"); } // ----- VUOTO -----
	if (colore==1) { gtk_widget_set_name (gwidget, "nero"); }   // ----- PLAYER 1 -----
	if (colore==2) { gtk_widget_set_name (gwidget, "rosso"); }  // ----- PLAYER 2 -----
	if (colore==3) { gtk_widget_set_name (gwidget, "giallo"); } // ----- MOSSA -----
	if (colore==4) { gtk_widget_set_name (gwidget, "viola"); }  // ----- MANGIA -----
	if (colore==5) { gtk_widget_set_name (gwidget, "verde"); }  // ----- SPECIAL -----
}


/* ***** RESET BUTTON***** */
void on_button65_clicked (GtkButton *button, gpointer user_data)
{
	vittorie[0]=0;
	vittorie[1]=0;
	gpk_game_reset(button);
	gpk_update_chessboard(button);
	gpk_update_stats(button);
	g_print ("Call Reset\n");
}


/* ***** 2 PLAYERS BUTTON***** */
void on_button66_clicked (GtkButton *button, gpointer user_data)
{
	gpk_update_stat_str(button, 24, "Play free chess");
	g_print ("Call 2 Players\n");
}


/* ***** 1 PLAYER BUTTON***** */
void on_button67_clicked (GtkButton *button, gpointer user_data)
{
	gpk_update_stat_str(button, 24, "You are black");
	g_print ("Call 1 Players\n");
	stato=20;
	if (thread_started==0) {
		pthread_create(&thd, NULL, gpk_thread, button);
		g_print ("Thread started now....\n");
		thread_started++;
	}
	g_print ("Thread informaed to start game....\n");
}


/* ***** 0 PLAYERS BUTTON ***** */
void on_button68_clicked (GtkButton *button, gpointer user_data)
{
	gpk_update_stat_str(button, 24, "Autoplay");
	g_print ("Call 0 Players\n");
	stato=10;
	if (thread_started==0) {
		pthread_create(&thd, NULL, gpk_thread, button);
		g_print ("Thread started now....\n");
		thread_started++;
	}
	g_print ("Thread informaed to start game....\n");
}


/* ***** AUTOPLAY THREAD ***** */
void gpk_thread(void *arg) {

	int x,y,r;
	GtkButton *button=(GtkButton *)arg;

	g_print ("Thread UP and RUNNING !!\n");
	while (1) {
		while ((stato==9)||(stato==10)||(stato==22)||(stato==23)) {
			g_print ("Hit....");
			valid_chip_cnt=gpk_find_chips(actual_player);
			if (valid_chip_cnt>0) {
				x=valid_chip[(int)(random()%valid_chip_cnt)][0];
				y=valid_chip[(int)(random()%valid_chip_cnt)][1];
				gdk_threads_enter();
				gpk_actor_game(x, y, button);
				gdk_threads_leave();
				usleep(50000);
				if (best_move_cnt>0) { // -------- cerca la prima mossa buona (o 1 a caso o la migliore)--------
					/*
					r=(int)(random()%best_move_cnt);
					gdk_threads_enter();
					gpk_actor_game(best_move[r][0], best_move[r][1], button);
					gdk_threads_leave();
					usleep(100000);
					*/
					gdk_threads_enter();
					gpk_actor_game(best_move[1][0], best_move[1][1], button);
					gdk_threads_leave();
					g_print (" GOOD move found !!\n");
				} else if (valid_move_cnt>0) { // -------- se non la trova ne fa una a caso delle valide --------
					r=(int)(random()%valid_move_cnt);
					gdk_threads_enter();
					gpk_actor_game(valid_move[r][0], valid_move[r][1], button);
					gdk_threads_leave();
					g_print (" valid found\n");
				} else {
					g_print (" miss\n");
				}
			} else {
				g_print ("NO CHIPS (You would not be here !!)\n");
			}
		}
		usleep(50000);
	}
	g_print ("THREAD END (You would not be here !!)\n");
}


/* ***** INTERCETTA CALLBACK SEGNALI ***** */
gboolean on_entry1_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { g_print ("Call GG\n"); return FALSE; }

gboolean on_entry26_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 0, widget); return FALSE; }
gboolean on_entry27_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 0, widget); return FALSE; }
gboolean on_entry28_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 0, widget); return FALSE; }
gboolean on_entry29_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 0, widget); return FALSE; }
gboolean on_entry30_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 0, widget); return FALSE; }
gboolean on_entry31_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(5, 0, widget); return FALSE; }
gboolean on_entry32_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(6, 0, widget); return FALSE; }
gboolean on_entry33_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(7, 0, widget); return FALSE; }

gboolean on_entry34_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(0, 1, widget); return FALSE; }
gboolean on_entry35_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(1, 1, widget); return FALSE; }
gboolean on_entry36_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(2, 1, widget); return FALSE; }
gboolean on_entry37_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(3, 1, widget); return FALSE; }
gboolean on_entry38_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(4, 1, widget); return FALSE; }
gboolean on_entry39_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(5, 1, widget); return FALSE; }
gboolean on_entry40_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(6, 1, widget); return FALSE; }
gboolean on_entry41_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(7, 1, widget); return FALSE; }

gboolean on_entry42_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 2, widget); return FALSE; }
gboolean on_entry43_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 2, widget); return FALSE; }
gboolean on_entry44_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 2, widget); return FALSE; }
gboolean on_entry45_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 2, widget); return FALSE; }
gboolean on_entry46_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 2, widget); return FALSE; }
gboolean on_entry47_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 2, widget); return FALSE; }
gboolean on_entry48_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 2, widget); return FALSE; }
gboolean on_entry49_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 2, widget); return FALSE; }

gboolean on_entry50_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 3, widget); return FALSE; }
gboolean on_entry51_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 3, widget); return FALSE; }
gboolean on_entry52_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 3, widget); return FALSE; }
gboolean on_entry53_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 3, widget); return FALSE; }
gboolean on_entry54_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 3, widget); return FALSE; }
gboolean on_entry55_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 3, widget); return FALSE; }
gboolean on_entry56_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 3, widget); return FALSE; }
gboolean on_entry57_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 3, widget); return FALSE; }

gboolean on_entry58_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 4, widget); return FALSE; }
gboolean on_entry59_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 4, widget); return FALSE; }
gboolean on_entry60_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 4, widget); return FALSE; }
gboolean on_entry61_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 4, widget); return FALSE; }
gboolean on_entry62_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 4, widget); return FALSE; }
gboolean on_entry63_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 4, widget); return FALSE; }
gboolean on_entry64_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 4, widget); return FALSE; }
gboolean on_entry65_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 4, widget); return FALSE; }

gboolean on_entry66_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 5, widget); return FALSE; }
gboolean on_entry67_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 5, widget); return FALSE; }
gboolean on_entry68_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 5, widget); return FALSE; }
gboolean on_entry69_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 5, widget); return FALSE; }
gboolean on_entry70_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 5, widget); return FALSE; }
gboolean on_entry71_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 5, widget); return FALSE; }
gboolean on_entry72_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 5, widget); return FALSE; }
gboolean on_entry73_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 5, widget); return FALSE; }

gboolean on_entry74_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 6, widget); return FALSE; }
gboolean on_entry75_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 6, widget); return FALSE; }
gboolean on_entry76_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 6, widget); return FALSE; }
gboolean on_entry77_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 6, widget); return FALSE; }
gboolean on_entry78_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 6, widget); return FALSE; }
gboolean on_entry79_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 6, widget); return FALSE; }
gboolean on_entry80_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 6, widget); return FALSE; }
gboolean on_entry81_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 6, widget); return FALSE; }

gboolean on_entry82_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 7, widget); return FALSE; }
gboolean on_entry83_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 7, widget); return FALSE; }
gboolean on_entry84_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 7, widget); return FALSE; }
gboolean on_entry85_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 7, widget); return FALSE; }
gboolean on_entry86_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 7, widget); return FALSE; }
gboolean on_entry87_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 7, widget); return FALSE; }
gboolean on_entry88_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 7, widget); return FALSE; }
gboolean on_entry89_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 7, widget); return FALSE; }

	/*
	int i,j;
	gchar temp[5];	
	
	gtk_rc_parse("/home/gipoco/Projects/project1/src/joshua.rc");
	
	for (i=0; i<8; i++) {
		for (j=0; j<8; j++) {
			sprintf(temp, "%d\n", (((i%2)>0)?(((j%2)>0)?1:0):(((j%2)>0)?0:1)));
			g_print (temp);
			gpk_set_checkboard(button, i, j, ' ', (((i%2)>0)?(((j%2)>0)?1:0):(((j%2)>0)?0:1)));
		}
		
	}
	*/

